<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

$componentPath = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
JLoader::register('NicepageHelpersNicepage', $componentPath . '/helpers/nicepage.php');
JLoader::register('NicepageThemeNicepage', $componentPath . '/helpers/theme.php');
require $componentPath . '/helpers/modules.php';

$app = JFactory::getApplication();
$doc = JFactory::getDocument();
$props = $app->get('props');
$editLinkHtml = $app->get('editLinkHtml');
$bodyClass = isset($props['bodyClass']) ? $props['bodyClass'] : '';
$bodyStyle = isset($props['bodyStyle']) ? $props['bodyStyle'] : '';
$head = isset($props['head']) ? $props['head'] : '';
$fonts = isset($props['fonts']) ? $props['fonts'] : '';
$pageId = isset($props['pageId']) ? $props['pageId'] : '';
$publishHtml = isset($props['publishHtml']) ? $props['publishHtml'] : '';
$publishHtml = NicepageHelpersNicepage::processSectionsHtml($publishHtml, true, $pageId);
$backlink = isset($props['backlink']) ? $props['backlink'] : '';
$pageCssUsedIds = isset($props['pageCssUsedIds']) ? $props['pageCssUsedIds'] : '';
$pageId = isset($props['pageId']) ? $props['pageId'] : '';
$hideHeader = isset($props['hideHeader']) ? $props['hideHeader'] : false;
$hideFooter = isset($props['hideFooter']) ? $props['hideFooter'] : false;
$isPreview = isset($props['isPreview']) ? $props['isPreview'] : false;
if (isset($props['titleInBrowser']) && $props['titleInBrowser'] != '') {
    $doc->setTitle($props['titleInBrowser']);
}

if (isset($props['description']) && $props['description'] != '') {
    $doc->setDescription($props['description']);
}

if (isset($props['keywords']) && $props['keywords'] != '') {
    $doc->setMetadata('keywords', $props['keywords']);
}

if (isset($props['metaTags']) && $props['metaTags'] != '') {
    $doc->addCustomTag($props['metaTags']);
}

if (isset($props['customHeadHtml']) && $props['customHeadHtml'] != '') {
    $doc->addCustomTag($props['customHeadHtml']);
}

JHtml::_('bootstrap.framework');

$params = NicepageHelpersNicepage::getConfig($isPreview);

$publishCookiesSection = '';
$cookiesConfirmCode = '';
if (isset($params['cookiesConsent'])) {
    $cookiesConsent = json_decode($params['cookiesConsent'], true);
    if ($cookiesConsent && (!$cookiesConsent['hideCookies'] || $cookiesConsent['hideCookies'] === 'false')) {
        $publishCookiesSection = $cookiesConsent['publishCookiesSection'];
        $cookiesConfirmCode = $cookiesConsent['cookieConfirmCode'];
    }
}

$css = NicepageHelpersNicepage::buildSiteStyleCss($params, $pageCssUsedIds, $publishHtml, $pageId);
$assets =  $doc->baseurl . '/components/com_nicepage/assets';
$publishPageParts = str_replace(
    '[[site_path_live]]',
    JURI::root(),
    array(
        'publishHtml'   => $publishHtml,
        'head'          => $head,
        'bodyStyle'     => $bodyStyle,
        'header'        => $params['header'],
        'footer'        => $params['footer'],
        'publishCookiesSection' => $publishCookiesSection,
        'fonts' => $fonts,
    )
);

$publishHeader = '';
if ($publishPageParts['header'] && !$hideHeader) {
    $headerItem = json_decode($publishPageParts['header'], true);
    if ($headerItem) {
        ob_start();
        echo $headerItem['styles'];
        echo $headerItem['php'];
        $publishHeader = ob_get_clean();
        $publishHeader = NicepageHelpersNicepage::processSectionsHtml($publishHeader, true, 'header');
    }
}

$publishFooter = '';
if ($publishPageParts['footer'] && !$hideFooter) {
    $footerItem = json_decode($publishPageParts['footer'], true);
    if ($footerItem) {
        ob_start();
        echo $footerItem['styles'];
        echo $footerItem['php'];
        $publishFooter = ob_get_clean();
        $publishFooter = NicepageHelpersNicepage::processSectionsHtml($publishFooter, true, 'footer');
    }
}


?>
<!DOCTYPE html>
<html>
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="stylesheet" type="text/css" media="all" href="<?php echo $assets; ?>/css/nicepage.css" id="nicepage-style-css">
    <link rel="stylesheet" type="text/css" media="all" href="<?php echo $assets; ?>/css/media.css" id="theme-media-css">
    <jdoc:include type="head" />
    <?php if (isset($params['jquery']) && $params['jquery'] == '1') : ?>
        <script type="text/javascript" src="<?php echo $assets; ?>/js/jquery.js"></script>
    <?php endif; ?>
    <script type="text/javascript" src="<?php echo $assets; ?>/js/nicepage.js"></script>
    <?php echo $publishPageParts['fonts']; ?>
    <style>
        <?php echo $css; ?>
        <?php echo $publishPageParts['head']; ?>
    </style>
    <?php echo $cookiesConfirmCode; ?>
</head>
<body class="<?php echo $bodyClass; ?>" style="<?php echo $publishPageParts['bodyStyle']; ?>">
<?php
echo $publishHeader;
?>
<?php echo $publishPageParts['publishHtml']; ?>
<?php
echo $publishFooter
?>
<?php echo $backlink; ?>
<?php echo $editLinkHtml; ?>
<?php echo $publishPageParts['publishCookiesSection']; ?>
</body>
</html>