<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

/**
 * Class NicepageController
 */
class NicepageController extends JControllerLegacy
{
    /**
     * Default display view
     *
     * @param bool $cachable
     * @param bool $urlparams
     *
     * @return mixed
     */
    public function display($cachable = false, $urlparams = false)
    {
        $input = JFactory::getApplication()->input;
        $uid = $input->get('uid', '');
        if ($uid) {
            $session = JFactory::getSession();
            $user = new JUser((int) $uid);
            $session->set('user', $user);
            exit(json_encode(array('result' => 'ok')));
        }

        return parent::display($cachable, $urlparams);
    }

    /**
     * Get active menu
     */
    public function menu()
    {
        JLoader::register('NicepageHelpersNicepage', JPATH_ADMINISTRATOR . '/components/com_nicepage/helpers/nicepage.php');
        JLoader::register('Nicepage_Data_Mappers', JPATH_ADMINISTRATOR . '/components/com_nicepage/tables/mappers.php');
        JLoader::register('ModMenuHelper', dirname(JPATH_ADMINISTRATOR) . '/modules/mod_menu/helper.php');

        $menuItemsMapper = Nicepage_Data_Mappers::get('menuItem');

        $home = $menuItemsMapper->find(array('home' => 1));
        if (count($home) < 1) {
            exit(json_encode(array('result' => null)));
        }

        $params = array(
            'menutype' => $home[0]->menutype,
            'startLevel' => '1',
            'endLevel' => '0',
            'showAllChildren' => '1',
            'tag_id' => '',
            'class_sfx' => '',
            'window_open' => '',
            'layout' => '_:default',
            'moduleclass_sfx' => '',
            'cache' => '1',
            'cache_time' => '900',
            'cachemode' => 'itemid'
        );
        $registry = new JRegistry();
        $registry->loadArray($params);
        $list = ModMenuHelper::getList($registry);

        if (count($list) < 1) {
            exit(json_encode(array('result' => null)));
        }

        $result = array();
        foreach ($list as $item) {
            $itemOptions = array (
                'title' => $item->title,
                'id'   => $this->_getPageId($item),
            );
            if ($item->level == 1) {
                $result[] = $itemOptions;
            } else {
                $lastIndex = count($result) - 1;
                $element = $result[$lastIndex];
                $result[$lastIndex] = $this->_addItemToResult($element, $item, $item->level);
            }
        }
        exit(json_encode(array('result' => $result)));
    }

    /**
     * Build nested structure
     *
     * @param array  $element Element
     * @param object $item    Menu Item
     * @param int    $level   Level
     *
     * @return mixed
     */
    private function _addItemToResult($element, $item, $level) {
        if ($level > 2) {
            $subel = end($element['items']);
            $element['items'][count($element['items']) - 1] = $this->_addItemToResult($subel, $item, --$level);
        } else {
            if (!isset($element['items'])) {
                $element['items'] = array();
            }
            $element['items'][] = array(
                'title' => $item->title,
                'id'   => $this->_getPageId($item),
            );
        }
        return $element;
    }

    /**
     * Get page id
     *
     * @param object $item Menu item
     *
     * @return int|string
     */
    private function _getPageId($item) {
        if (preg_match('/index.php\?option=com_content&view=article&id=(\d+)/', $item->link, $matches)) {
            $page = NicepageHelpersNicepage::getSectionsTable();
            return $page->load(array('page_id' => $matches[1])) ? (int)$page->id : '';
        } else {
            return '';
        }
    }

    /**
     * Send mail to site owner.
     */
    public function form()
    {
        $input = JFactory::getApplication()->input;
        $formId = $input->get('formId', '');
        $pageId = $input->get('id', '');
        if (!$formId || !$pageId) {
            exit(json_encode(array('error' => 'Form id or page id not found')));
        }
        $formsData = null;
        $helpersDir = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers';
        JLoader::register('NicepageHelpersNicepage', $helpersDir . '/nicepage.php');
        if ($pageId == 'header' || $pageId == 'footer') {
            $config = NicepageHelpersNicepage::getConfig();
            if (isset($config[$pageId]) && $config[$pageId]) {
                $item = json_decode($config[$pageId], true);
                $formsData = isset($item['formsData']) ? json_decode($item['formsData'], true) : array();
            }
        } else {
            $page = NicepageHelpersNicepage::getSectionsTable();
            if ($page->load(array('page_id' => $pageId))) {
                $props = NicepageHelpersNicepage::decodeProps($page->props);
                $formsData = isset($props['formsData']) ? json_decode($props['formsData'], true) : array();
            }
        }

        if ($formsData) {
            $foundForm = null;
            for ($i = 0; $i < count($formsData); $i++) {
                $form = $formsData[$i];
                $str = json_encode($form);
                if (strpos($str, 'form-' . $formId) !== false) {
                    $foundForm = $form;
                    break;
                }
            }
            if ($foundForm) {
                $convertedForm = array(
                    'subject' => $foundForm['subject'],
                    'email_message' => $foundForm['emailMsg'],
                    'success_redirect' => '',
                    'email' => array(
                        'from' => $foundForm['emailfrom'],
                        'to' => $foundForm['emailto']
                    ),
                    'fields' => array(),
                );
                for ($j = 0; $j < count($foundForm['fields']); $j++) {
                    $field = $foundForm['fields'][$j];
                    $convertedForm['fields'][$field['name']] = array(
                        'order' => $field['order'],
                        'type' => $field['type'],
                        'label' => $field['label'],
                        'required' => $field['required'],
                        'errors' => array(
                            'required' => 'Field \'' . $field['label'] . '\' is required.'
                        )
                    );
                }

                $formsDir = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers/forms/';
                JLoader::register('FormProcessor', $formsDir . '/FormProcessor.php');
                $processor = new FormProcessor();
                $processor->process($convertedForm);
                exit;
            }
        }
    }

    /**
     * Send mail to site owner.
     */
    public function sendmail()
    {
        $config = JFactory::getConfig();

        $recipient = $config->get('mailfrom');

        $input = JFactory::getApplication()->input;

        if ($input->exists('recaptchaResponse')) {
            $response = $input->get('recaptchaResponse', '', 'RAW');
            $helpersDir = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers';
            JLoader::register('NicepageHelpersNicepage', $helpersDir . '/nicepage.php');
            JLoader::register('NicepageReCaptcha', $helpersDir . '/recaptcha.php');
            $config = NicepageHelpersNicepage::getConfig();
            if (isset($config['siteSettings'])) {
                $settings = json_decode($config['siteSettings'], true);
                if (isset($settings['captchaSecretKey']) && $settings['captchaSecretKey']) {
                    $recaptcha = new NicepageReCaptcha($settings['captchaSecretKey']);
                    $result = $recaptcha->verifyResponse($response);
                    if (!$result->success) {
                        // Not verified - show form error
                        $error = is_array($result->errorCodes) ? implode(' ', $result->errorCodes) : $result->errorCodes;
                        echo json_encode(array('error' => $error));
                        exit;
                    }
                }
            }
        }

        $body = '';

        $excludeKeys = array('recaptchaResponse', 'Itemid', 'redirect', 'siteId', 'pageId');
        $data = $input->getArray($_POST);
        $subject = '';
        foreach ($data as $key => $value) {
            if (array_search($key, $excludeKeys) === false && $data[$key]) {
                if (!$subject && ($key == 'name' || strpos($key, 'name') !== false)) {
                    $subject = $data[$key];
                }
                $returnValues = '';
                if (is_array($data[$key])) {
                    foreach ($data[$key] as $k => $v) {
                        $returnValues .= $v;
                        if ($k !== count($data[$key]) - 1) {
                            $returnValues .= ', ';
                        }
                    }
                } else {
                    $returnValues = $data[$key];
                }
                $body .= ucfirst($key) . ": " . $returnValues . "\n";
            }
        }
        if (!$subject) {
            $subject = 'Mail subject';
        }

        $redirect = $input->get('redirect', '', 'string');

        $mail = JFactory::getMailer();
        $mail->setSubject($subject);
        $mail->setBody($body);

        if (filter_var($recipient, FILTER_VALIDATE_EMAIL)) {
            $mail->addRecipient($recipient);
        }

        $ret = $mail->Send();

        if ($redirect) {
            JFactory::getApplication()->redirect($redirect);
        } else {
            header('Content-Type: application/json');
            $data = array();
            if ($ret) {
                $data['success'] = true;
            } else {
                $data['error'] = $ret;
            }
            echo json_encode($data);
            exit;
        }
    }
}
