<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

/**
 * Class PlgContentNicepage
 */
class PlgSystemNicepage extends JPlugin
{
    /**
     * Process component content
     */
    public function onAfterDispatch()
    {
        $app = JFactory::getApplication();

        if ($app->isAdmin() || ($app->get('offline') && !JFactory::getUser()->authorise('core.login.offline'))) {
            return;
        }

        $doc = JFactory::getDocument();
        $buf = $doc->getBuffer('component');
        $reLanding = '/<\!--np\_landing-->([\s\S]+?)<\!--\/np\_landing-->/';
        if (preg_match($reLanding, $buf, $landingMatches)) {
            $result = unserialize(base64_decode($landingMatches[1]));
            $app->set('theme', 'landing');
            $app->set('themes.base', JPATH_ADMINISTRATOR . '/components/com_nicepage/views');
            $app->set('themeFile', 'landing.php');
            $app->set('props', $result['props']);
            $app->set('editLinkHtml', $result['editLinkHtml']);
        }
    }

    /**
     *  Proccess page content after rendering
     */
    public function onAfterRender()
    {
        $app = JFactory::getApplication();
        $isAdmin = $app->isAdmin();

        $pageContent = $app->getBody();

        // Move dataBridge object initialization at top head tag in admin panel
        if ($isAdmin && preg_match('/<\!--np\_databridge_script-->([\s\S]+?)<\!--\/np\_databridge_script-->/', $pageContent, $adminScriptsMatches)) {
            $adminPageScripts = $adminScriptsMatches[1];
            $pageContent = str_replace($adminScriptsMatches[0], '', $pageContent);
            $pageContent = preg_replace('/(<head>)/', '$1[[dataBridgeScript]]', $pageContent, 1);
            $pageContent = str_replace('[[dataBridgeScript]]', $adminPageScripts, $pageContent);
            $app->setBody($pageContent);
            return;
        }

        if ($isAdmin) {
            return;
        }

        // Get styles, script, backlink from article content
        $pageStyles = $this->_getStyles($pageContent);
        $pageScripts = $this->_getScripts($pageContent);
        $pageBacklink = $this->_getBacklink($pageContent);
        $pageCookiesConsent = $this->_getCookiesConsent($pageContent);
        $pageCookiesConfirmCode = $this->_getCookiesConfirmCode($pageContent);

        // Process article content by placeholder
        if (preg_match('/<\!--np\_content-->([\s\S]+?)<\!--\/np\_content-->/', $pageContent, $matches)) {
            $sectionsHtml = $matches[1];
            $reView = '/<\!--np\_view-->([\s\S]+?)<\!--\/np\_view-->/';
            $pageView = 'default';
            if (preg_match($reView, $pageContent, $viewMatches)) {
                $pageView = $viewMatches[1];
                $pageContent = preg_replace($reView, '', $pageContent);
            }
            if ($sectionsHtml && $pageView === 'landing_with_header_footer') {
                $bodyStartTag = '<body>';
                $bodyContent = '';
                $bodyEndTag = '</body>';
                if (preg_match('/(<body[^>]+>)([\s\S]*)(<\/body>)/', $pageContent, $matches)) {
                    $bodyStartTag = $matches[1];
                    $bodyContent = trim($matches[2]);
                    $bodyEndTag = $matches[3];
                }

                if ($bodyContent == '') {
                    $newPageContent = $bodyStartTag . $sectionsHtml . $bodyEndTag;
                } else {
                    $newPageContent = $bodyStartTag;
                    if (preg_match('/<header[^>]+>[\s\S]*<\/header>/', $bodyContent, $matches2)) {
                        $newPageContent .= $matches2[0];
                    }
                    $newPageContent .= $sectionsHtml;
                    if (preg_match('/<footer[^>]+>[\s\S]*<\/footer>/', $bodyContent, $matches3)) {
                        $newPageContent .= $matches3[0];
                    }
                    if (preg_match('/<\/footer>([\s\S]*)/', $bodyContent, $matches4)) {
                        $newPageContent .= $matches4[1];
                    }
                    $newPageContent .= $bodyEndTag;
                }
                $pageContent = preg_replace('/(<body[^>]+>)([\s\S]*)(<\/body>)/', '[[body]]', $pageContent);
                $pageContent = str_replace('[[body]]', $newPageContent, $pageContent);
            } else {
                $pageContent = preg_replace('/<!--\/?np\_content-->/', '', $pageContent);
            }
        }
        if ($app->input->get('toEdit', '0') === '1') {
            $pageContent = preg_replace('/class="(item-page|u-page-root)/', ' id="np-test-container" class="$1', $pageContent);
        }
        $pageContent = str_replace('</head>', $pageStyles . '</head>', $pageContent);
        $pageContent = str_replace('</head>', $pageScripts . '</head>', $pageContent);
        $pageContent = str_replace('</head>', $pageCookiesConfirmCode . '</head>', $pageContent);
        $pageContent = str_replace('</body>', $pageBacklink . '</body>', $pageContent);
        $pageContent = str_replace('</body>', $pageCookiesConsent . '</body>', $pageContent);

        $settings = $this->getSettings();
        if (isset($settings['captchaScript']) && $settings['captchaScript'] && strpos($pageContent, 'recaptchaResponse') !== false) {
            $pageContent = str_replace('</head>', $settings['captchaScript'] . '</head>', $pageContent);
        }
        if (strpos($pageContent, '<meta name="viewport"') === false) {
            $pageContent = str_replace('<head>', '<head>' . '<meta name="viewport" content="width=device-width, initial-scale=1.0" />', $pageContent);
        }
        if (isset($settings['metaTags']) && $settings['metaTags'] && strpos($pageContent, $settings['metaTags']) === false) {
            $pageContent = str_replace('</head>', $settings['metaTags'] . '</head>', $pageContent);
        }
        if (isset($settings['headHtml']) && $settings['headHtml'] && strpos($pageContent, $settings['headHtml']) === false) {
            $pageContent = str_replace('</head>', $settings['headHtml'] . '</head>', $pageContent);
        }
        if (isset($settings['analyticsCode']) && $settings['analyticsCode'] && strpos($pageContent, $settings['analyticsCode']) === false) {
            $pageContent = str_replace('</head>', $settings['analyticsCode'] . '</head>', $pageContent);
        }
        if (isset($settings['keywords']) && $settings['keywords'] && strpos($pageContent, $settings['keywords']) === false) {
            if (preg_match('/<meta\s+?name="keywords"\s+?content="([^"]+?)"\s+?\/>/', $pageContent, $keywordsMatches)) {
                $pageContent = str_replace($keywordsMatches[0], '<meta name="keywords" content="' . $settings['keywords'] . ', ' . $keywordsMatches[1] . '" />', $pageContent);
            } else {
                $pageContent = str_replace('<title>', '<meta name="keywords" content="' . $settings['keywords'] . '" />' . '<title>', $pageContent);
            }
        }
        if (isset($settings['description']) && $settings['description'] && strpos($pageContent, $settings['description']) === false) {
            if (preg_match('/<meta\s+?name="description"\s+?content="([^"]+?)"\s+?\/>/', $pageContent, $descMatches)) {
                $pageContent = str_replace($descMatches[0], '<meta name="description" content="' . $settings['description'] . ', ' . $descMatches[1] . '" />', $pageContent);
            } else {
                $pageContent = str_replace('<title>', '<meta name="description" content="' . $settings['description'] . '" />' . '<title>', $pageContent);
            }
        }

        $app->setBody($pageContent);
    }

    /**
     * Get settings
     *
     * @return array
     */
    public function getSettings() {
        JLoader::register('NicepageHelpersNicepage', dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers/nicepage.php');
        $config = NicepageHelpersNicepage::getConfig();
        return isset($config['siteSettings']) ? json_decode($config['siteSettings'], true) : array();
    }

    /**
     * Process site styles
     *
     * @param string $pageContent Article content
     *
     * @return string
     */
    private function _getStyles(&$pageContent)
    {
        $pageStyles = '';
        if (preg_match('/<\!--np\_styles-->([\s\S]+?)<\!--\/np\_styles-->/', $pageContent, $stylesMatches)) {
            $pageStyles = $stylesMatches[1];
            $pageContent = str_replace($stylesMatches[0], '', $pageContent);
        }
        return $pageStyles;
    }

    /**
     * Process site scripts
     *
     * @param string $pageContent Article content
     *
     * @return string
     */
    private function _getScripts(&$pageContent)
    {
        $pageScripts = '';
        if (preg_match('/<\!--np\_scripts-->([\s\S]+?)<\!--\/np\_scripts-->/', $pageContent, $scriptsMatches)) {
            $pageScripts = $scriptsMatches[1];
            $pageContent = str_replace($scriptsMatches[0], '', $pageContent);
        }
        return $pageScripts;
    }

    /**
     * Process site backlink
     *
     * @param string $pageContent Article content
     *
     * @return mixed
     */
    private function _getBacklink(&$pageContent)
    {
        $pageBacklink = '';
        if (preg_match('/<\!--np\_backlink-->([\s\S]+?)<\!--\/np\_backlink-->/', $pageContent, $backlinkMatches)) {
            $pageBacklink = $backlinkMatches[1];
            $pageContent = str_replace($backlinkMatches[0], '', $pageContent);
        }
        return $pageBacklink;
    }

    /**
     * Process site cookies consent
     *
     * @param string $pageContent Article content
     *
     * @return mixed
     */
    private function _getCookiesConsent(&$pageContent)
    {
        $pageCookiesConsent = '';
        if (preg_match('/<\!--np\_cookies-->([\s\S]+?)<\!--\/np\_cookies-->/', $pageContent, $cookiesMatches)) {
            $pageCookiesConsent = $cookiesMatches[1];
            $pageContent = str_replace($cookiesMatches[0], '', $pageContent);
        }
        return $pageCookiesConsent;
    }

    /**
     * Process site cookies confirm code
     *
     * @param string $pageContent Article content
     *
     * @return mixed
     */
    private function _getCookiesConfirmCode(&$pageContent)
    {
        $pageCookiesConfirmCode = '';
        if (preg_match('/<\!--np\_cookiesconfirmcode-->([\s\S]+?)<\!--\/np\_cookiesconfirmcode-->/', $pageContent, $cookiesConfirmCodeMatches)) {
            $pageCookiesConfirmCode = $cookiesConfirmCodeMatches[1];
            $pageContent = str_replace($cookiesConfirmCodeMatches[0], '', $pageContent);
        }
        return $pageCookiesConfirmCode;
    }
}